/*
           panteltje mcam internet connection routines

         Copyright (C) 1999, 2000, 2001  Jan Mourer

 email: jan@panteltje.demon.nl
 snail mail:
 Jan Mourer
 Monnikebildtdijk 2
 9078 VE Oude Bildtzijl
 Holland

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "vvl_defs.h"
#include "internet.h"
#include "mcam.h"


extern int debug_flag;
extern int accept_internet_connections;
extern BYTE __huge *DataBuffer;
extern BYTE __huge *gStreamBuffer;
extern char *server_password;
extern char *server_control_password;
extern int start_frame, frame_count, frame_delay, save_frames_flag;
extern int save_frames_format;
extern int picture_format;
extern int output_to_frame_flag;
extern char *frame_dir;
extern int store_only_on_change_flag;
extern int coding_mode;
extern int coding_quality;
extern char *home_dir;
extern int gain_ceiling;
extern int man_exposure_flag;
extern int color_balance_mode; /* 1 = auto */
extern char *command_line_preset_filename;
extern int clock_position;
extern int use_utc_flag;
extern int show_time_flag;
extern char screen_text[];
extern int text_to_screen_flag;
extern int text_x, text_y;
extern int text_fr, text_fg, text_fb, text_br, text_bg, text_bb;
extern int selected_font;
extern int color_depth;
extern int color_depth_mode;

int init_sockaddr (\
struct sockaddr_in *name, const char *hostname, unsigned short int port)
{
struct hostent *hostinfo;
     
fprintf(stderr, "Getting host %s port %d by name.\n",\
hostname, port);

name -> sin_family = AF_INET;
name -> sin_port = htons (port);
hostinfo = gethostbyname (hostname);
if (hostinfo == NULL)
	{
	fprintf (stderr, "Unknown host %s.\n", hostname);
	return 0;
	}
name -> sin_addr = *(struct in_addr *) hostinfo -> h_addr;
}/* end function init_sockaddr */


int read_from_server(\
int argc, char **argv, char *server, int port, char *password)
{
int a, b, c, i, b0, b1;
int x, y;
int sock;
struct sockaddr_in servername;
int bytes_read;
int bytes_written;
int bytes_received;
int frames;
unsigned char *ucbufptr;
int buffer_length;
int expected_length;
int header_read_flag;
time_t start_time, elapsed_time;
float frames_per_second;
unsigned long total_bytes_received;
float bytes_per_second;
int message_number;
int old_message_number;
fd_set active_fd_set, read_fd_set;
struct timeval timeout;
char txbuffer[HEADER_SIZE]; /* password, text, command */
int confirm_msg_send_flag;
char temp[HEADER_SIZE]; /* text size is always < HEADER_SIZE */
int show_camera_status_flag;
int compression_flag;
char temp2[4096];
FILE *fptr;
int command_file_open_flag;
int command_in_progress_flag;
int save_status_flag;
FILE *save_fptr;

/* Create the socket. */
sock = socket (PF_INET, SOCK_STREAM, 0);
if (sock < 0)
	{
	perror ("read_from_server(): socket (client)");
    return 0;
	}
     
/* Connect to the server. */
init_sockaddr (&servername, server, port);
a = connect (sock, (struct sockaddr *) &servername, sizeof (servername) );
if(a < 0)
	{
	perror ("read_from_server(): connect (client)");
	close(sock);
	return 0;
	}     

/* Initialize the set of active sockets. */
FD_ZERO (&active_fd_set);

/* listen to socket */
FD_SET (sock, &active_fd_set);

/* listen to user for possible abort */
FD_SET(fileno(stdin), &active_fd_set);     

/* clear txbuffer */
for(i = 0; i < HEADER_SIZE; i++)
	{
	txbuffer[i] = 0;
	}

/* password to txbuffer */
strcpy(txbuffer + START_PASSWORD, password);

fprintf(stderr,\
"Reading data from %s port %d\n", server, port);

fprintf(stderr, \
"lines starting with a '/' are commands\n\
/quit to abort\n\
/stat for statistics\n\
/help for a list commands\n");

fprintf(stderr,\
"Any other text (max %d characters) is send to the server\n\
After typing a line of text, wait for 'msg send'!\n",\
START_COMMAND - START_CLIENT_TEXT);
if(output_to_frame_flag)
	{
	fprintf(stderr,\
"HANDSHAKE OUTPUT MODE 'O' ACTIVATED,\n\
mcam will now wait for %s/mcam.ppm.ready to be deleted,\n\
before it will read the next frame into %s/mcam.ppm,\n\
and creates a new %s/mcam.ppm.ready.\n\
ctrl C aborts\n",\
	frame_dir, frame_dir); 
	}
/* read continiously frames from server */
old_message_number = -1;
elapsed_time = 0;
start_time = time(0);
total_bytes_received = 0;
frames = 0;
show_camera_status_flag = 0;
save_status_flag = 0;
expected_length = 0;
/* send tx buffer, password plus text */
bytes_written = write (sock, txbuffer, HEADER_SIZE);
if (bytes_written < 0)
	{
	fprintf(stderr, "could not send txbuffer\n");
	perror ("write");
	close(sock);
	return 0;
	}

command_file_open_flag = 0;
command_in_progress_flag = 0;
/* monitor for any activity on socket and stdin */
while(1)
	{
	/* Block until input arrives on one or more active sockets. */
	read_fd_set = active_fd_set;

	timeout.tv_sec = 1;
	timeout.tv_usec = 0;
	a = select (FD_SETSIZE, &read_fd_set, NULL, NULL, &timeout);
	if(a < 0)
		{
		perror ("select");
		return 0;
		}
	if(a == 0)/* timeout */     
		{
/*		fprintf(stderr, "client select timeout\n");	*/
		continue;
		}

	/* command flag */
	b = 0;

	/* test for preset file specified in command line */
	if (command_line_preset_filename)
		{
		/* temp2 is used for reporting ready later */
		strcpy(temp2, command_line_preset_filename);
		
		/* try to open file for read */
		sprintf(temp, "%s/p.%s", frame_dir, temp2);
		fptr = fopen(temp, "r");
		if(! fptr)
			{
			fprintf(stderr, "Cannot open file %s for read\n", temp2);

			/* only execute this preset file ONCE */
			free(command_line_preset_filename);
			command_line_preset_filename = NULL;		
			strerror(errno);
			
			continue;
			}
		
		/* only execute this preset file ONCE */
		free(command_line_preset_filename);
		command_line_preset_filename = NULL;		

		FD_SET(fileno(fptr), &active_fd_set);     

		command_file_open_flag = 1;
			
		/* report to user what is happening */
		fprintf(stderr,"Starting processing of command file %s\n",\
		temp2);
			
		continue;
		}

	/* test for command file open */
	if( (! command_in_progress_flag) && command_file_open_flag)
		{
		if(FD_ISSET (fileno(fptr), &read_fd_set) )
			{
			/* clear the command array */
			temp[0] = 0;

			/* for all characters in a line */
			i = 0;
			while(1) 
				{
				while(1)/* try getc */
					{
					errno = 0;
					c = getc(fptr);
					if(errno == EAGAIN) continue;
					if(errno == EINTR) continue;
					break;
					}
				if(c == EOF)
					{
					FD_CLR (fileno(fptr), &active_fd_set);
					fclose(fptr);
				
					command_file_open_flag = 0;

					/* report to user what is happening */
					fprintf(stderr, "Ready processing command file %s\n",\
					temp2);

					/* break from line (later from file) */
					break;
					}
				
				/* test for end of line */
				if(c == '\n')
					{
					if(debug_flag)
						{
						fprintf(stderr,\
						"read returned a=%d, temp=%s strlen(temp)=%d\n",\
						a, temp, strlen(temp) );
						}
					
					/* mark end of text */
					temp[i] = 0;

					/* break from line, and execute command in temp */
					break;
					}/* end if '\n' */

				temp[i] = c;
				i++;

				/*
				signal command in progress, have to wait untill
				this flag is reset, before the next command from this
				file or console can be send
				*/
				command_in_progress_flag = 1;
				}/* for each character in a line */

			/* break from file, and test other input if EOF */
			if(c == EOF) continue;

			/* scip this line if it is a comment (started with '#') */
			if(temp[0] == '#') continue;

			/* scip this line if user attempts recursive */
			if(strncasecmp(temp, "/load", 5) == 0)
				{
				fprintf(stderr,\
"WARNING load command cannot be used recursive in command file, ignored:\n\
line reads\n\
%s\n",
temp);
				continue;
				}
			
			/* report to user what is happening */
			fprintf(stderr,\
			"sending command %s\n", temp);

			/* signal valid data */
			b = 1;
			}/* end if FD_ISSET */	
		}/* end if command file open flag */

	/* check if user action */
	if(FD_ISSET (fileno(stdin), &read_fd_set) )
		{
		a = read (fileno(stdin), temp, START_COMMAND - START_CLIENT_TEXT);
		/* string termination leave LF at end, to signal empty line */
		*(temp + a) = 0;
		if(debug_flag)
			{
			fprintf(stderr,\
			"read returned a=%d, temp=%s strlen(temp)=%d temp[0]=%d\n",\
			a, temp, strlen(temp), temp[0]);
			}
		
		/* signal valid data */
		b = 1;
		}

	/* if command file or user data */
	if(b)	
		{
		/* test for user commands */
		if(temp[0] != '/')
			{
			/* no user command, must be message text */

			/* message to tx buffer */
			strcpy(txbuffer + START_CLIENT_TEXT, temp);

			/* prevent text into command area */
			txbuffer[START_COMMAND] = 0;

			/* request confirm command if buffer is send */
			confirm_msg_send_flag = 1;

			continue;
			}/* end if text message */
			
		/* must be a command */
		/* sort commands */
		if(strncasecmp(temp, "/quit", 5) == 0)
			{
			fprintf(stderr,\
			"\nread_from_server(): user abort.\n");

			/* stop listening to socket */
			FD_CLR (sock, &active_fd_set);

			/* close socket */
			close(sock);

			return 0; 
			}/* end if /quit */
		else if(strncasecmp(temp, "/stat", 5) == 0)
			{
			fprintf(stderr,\
			"local: frame=%d  f/s=%.2f  KB/s=%.2f  B/f=%d\n"\
			,frames, frames_per_second, bytes_per_second / 1024.0, expected_length);

			/* signal to show status after the data has been received */
			show_camera_status_flag = 1;

			continue;
			}/* end if /stat */
		/* test if command for server camera */
		else if(strncasecmp(temp, "/camera", 7) == 0)
			{
			/* will return 0 if no command permission from server */
			if(construct_camera_command(temp + 8, txbuffer + START_COMMAND) )
				{
				/* request confirm command if buffer is send */
				confirm_msg_send_flag = 1;
				}

			continue;
			}/* end if command for server camera */			
		else if(strncasecmp(temp, "/help", 5) == 0)
			{
			show_commands();

			continue;
			}
		else if (strncasecmp(temp, "/local", 6) == 0)
			{
			execute_local_commands(temp + 7);
			
			continue;
			}
		else if (strncasecmp(temp, "/load", 4) == 0)
			{
			/* activate process from file */
			/* parse input */
			temp2[0] = 0;
			sscanf(temp, "/load %s", temp2);

			/* try to open file for read */
			sprintf(temp, "%s/p.%s", frame_dir, temp2);
			fptr = fopen(temp, "r");
			if(! fptr)
				{
				fprintf(stderr, "Cannot open file %s for read\n", temp2);
				strerror(errno);
				continue;
				}
		
			FD_SET(fileno(fptr), &active_fd_set);     

			command_file_open_flag = 1;
			
			/* report to user what is happening */
			fprintf(stderr,"Starting processing of command file %s\n",\
			temp2);
			
			continue;
			}
		else if(strncasecmp(temp, "/save", 5) == 0)
			{
			/* parse input */
			temp2[0] = 0;
			sscanf(temp, "/save %s", temp2);

			/* combine */
			sprintf(temp, "%s/p.%s", frame_dir, temp2);
			
			/* check if file already exists */
			save_fptr = fopen(temp, "r");
			if(save_fptr)
				{
				fclose(save_fptr);
				fprintf(stderr,\
				"File %s already exists, overwrite (y/n)? \n",\
				temp);
				fscanf(stdin, "%s", temp2);
				if(toupper(temp2[0]) != 'Y')
					{
					fprintf(stderr, "Save command cancelled\n");
					continue;
					}					
				}
			/* try to open file for write */
			save_fptr = fopen (temp, "w");
			if(! save_fptr)
				{
				fprintf(stderr,\
				"Could not open file %s for write\n", temp2);
				strerror(errno);
				continue;
				}

			/* report to user what is happening */
			fprintf(stderr, "saving status in file %s\n", temp);

			/* signal to save status after the data has been received */
			save_status_flag = 1;
			
			continue;
			}/* end if /stat */
		else
			{
			fprintf(stderr, "Unknown command\n");

			continue;
			}
		}/* end something on stdin */

	/* check if socket has data */
	if (FD_ISSET (sock, &read_fd_set) )
		{
		/* read from socket until header plus frame received */
		/*
		Note:
		first transmission header, then framebuffer, which has its
		own 64 byte header.
		Yes this system is camera dependend, so only for cpia.
		*/	
		expected_length = HEADER_SIZE; /* read at least the header */
		header_read_flag = 0;
		bytes_received = 0;
		ucbufptr = DataBuffer;
		buffer_length = STREAM_BUFFER_LEN + HEADER_SIZE;
		while(1)
			{
			/* read the data */
			errno = 0;
			if(debug_flag)
				{
				fprintf(stderr,\
				"read_from_server():\n\
 going to read sock=%d ucbufptr=%lu buffer_length=%d\n",\
				sock, (long)ucbufptr, buffer_length);
				}
			/* some parameter checking */
			if(bytes_received >= buffer_length)
				{
				/* something went wrong */
				fprintf(stderr,\
				"read_from_server():\n\
 bytes_received=%d buffer_length=%d aborting\n",\
				bytes_received, buffer_length);
				return 0;
				}
			bytes_read =\
			read(sock, ucbufptr, buffer_length - bytes_received);
			if (bytes_read < 0)
				{
				/* Read error. */
				perror ("read_from_server(): read");
				if(errno == EAGAIN) continue;
				if(errno == EINTR) continue;
				return 0;
				}
			else if (bytes_read == 0)/* EOF, connection closed by server */
				{
				/* we are here for example if incorrect password */
				fprintf(stderr, "read_from_server(): EOF detected\n");

				/* do not close socket */

/* maybe close socket here */
				return 0;
				}
		
			/* Data read. */
			ucbufptr += bytes_read;
			bytes_received += bytes_read;
			if(debug_flag)
				{
				fprintf (stderr,\
				"read_from_server():\n\
	bytes_read=%d bytes_received=%d expected_length=%d\n",\
				bytes_read, bytes_received, expected_length);
				}

			/* test if header complete */
			if(bytes_received >= HEADER_SIZE) 
				{
				if(! header_read_flag)
					{
					expected_length =\
					(DataBuffer[DATA_SIZE_VERY_HIGH] * 65536) +\
					(DataBuffer[DATA_SIZE_HIGH] * 256) +\
					DataBuffer[DATA_SIZE_LOW];
					if(debug_flag)
						{
						fprintf(stderr,\
						"read_from_server():\n\
 expected_length=%d after %d bytes_received\n\
 total_bytes_received=%lu\n",\
						expected_length, bytes_received,\
						total_bytes_received);
						}

					/* test if some text from server */
	
					/* get current message number */
					message_number =\
					DataBuffer[TEXT_MSG_NUMBER_HIGH] * 256 +\
					DataBuffer[TEXT_MSG_NUMBER_LOW];
				
					/* test for new message */
					if(message_number != old_message_number)
						{
						fprintf(stderr,\
						"Msg %d: %s",\
						message_number, DataBuffer + START_SERVER_TEXT);

						/* if text does not end with '\n' append one */
						a = strlen(DataBuffer + START_SERVER_TEXT);
						if( *(DataBuffer + START_SERVER_TEXT + a - 1) !=\
						'\n')
							{
							fprintf(stderr, "\n");
							}
						old_message_number = message_number;
						}/* end if new message number */
					
					/* test if camera status should be displayed */
					if(show_camera_status_flag)
						{
						show_camera_status(DataBuffer);
						show_camera_status_flag = 0;
						}

					/* test if status should be saved */
					if(save_status_flag)
						{
						save_status(DataBuffer, save_fptr);
						save_status_flag = 0;
						}
					
					header_read_flag = 1;
					}
				}/* end if bytes_received >= HEADER_SIZE */

			/* test if all data received */
			if(bytes_received == expected_length) break;
			}/* end while read from socket */
		if(debug_flag)
			{
			fprintf(stderr,\
			"read_from_server(): out while bytes_received=%d\n",\
			bytes_received);
			}

		/*
		data is in gStreamBuffer, which points to DataBuffer + HEADER_SIZE.
		*/
		/*
		test if any data, (server sends zero length if grab frame failed)
		*/
		if(expected_length != 0)
			{
			b0 = DataBuffer[FLAGS] & 2;
			b1 = DataBuffer[FLAGS] & 8;

			if( !b1 && !b0 ) coding_mode = NO_CODING;
			if( !b1 && b0 ) coding_mode = JPEG_CODING;
			if( b1 && !b0 ) coding_mode = MPEG2_CODING;
			if( b1 && b0 ) coding_mode = WAVELET_CODING;

			compression_flag = DataBuffer[HEADER_SIZE + 28];
			if(! display_a_frame(\
			argc, argv, expected_length, coding_mode, compression_flag) )
				{
				fprintf(stderr, "read_from_server(): could not display frame\n");
/* we do not want to exit here, so we can try camera retart */

/*				close (sock);*/
/*				return 0;*/
				}
			}/* end any data */
		frames++;

		elapsed_time = time(0) - start_time;
		frames_per_second = (float)frames / (float)elapsed_time;

		/* prevent divide by zero if here within one second */
		if(elapsed_time == 0) elapsed_time = 1;

		total_bytes_received += (unsigned long)bytes_received;
		bytes_per_second =\
		(float)total_bytes_received / (float)elapsed_time;

		/* send header */
		/*
		this will send the password, which will request the next frame,
		and send any text messages and commands.
		*/
		bytes_written = write (sock, txbuffer, HEADER_SIZE);
		if (bytes_written < 0)
			{
			fprintf(stderr, "could not send txbuffer\n");
			perror ("write");
			close(sock);
			return 0;
			}

		/* messagge send, mark message buffer empty */ 
		txbuffer[START_CLIENT_TEXT] = 0;/* text string starts here */
		
		/* command send, mark no command */
		txbuffer[START_COMMAND] = 0;

		/* print the prompt */
		if(confirm_msg_send_flag)
			{
			fprintf(stderr, "Msg send\n");
			confirm_msg_send_flag = 0;
			}

		/* reset command_in_progress_flag */
		command_in_progress_flag = 0;
		}/* end socket has data */
	}/* end while receiving frames */
return 1;
}/* end function read_from_server */


int construct_camera_command(char *text, BYTE __huge *buffer)
{
/*
the format is:
cmd, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8
cmd is loclated at buffer[0], arg1 atb buffer[1], etc...
text is any text after '/camera '
*/
int a;
char cmd[80];
char arg1[80];
char arg2[80];
char arg3[80];
char arg4[80];
char arg5[80];
char arg6[80];
char arg7[80];
char arg8[80];
char arg9[80];
int brightness, contrast, saturation;

if(debug_flag)
	{
	fprintf(stderr, "construct_camera_command(): arg text=%s buffer=%lu\n",\
	text, buffer);
	}

if(! text) return 0;
if(! buffer) return 0;

/* test if command permission aknowledged by server */
if( ! (*(DataBuffer + SERVER_STATUS + 36) & 1) )
	{
	fprintf(stderr,\
	"You have no command permission from server (password?).\n");	
	return 0;
	}

/* clear arguments */
cmd[0] = arg1[0] = arg2[0] = arg3[0] = arg4[0] = arg5[0] = arg6[0] =\
arg7[0] = arg8[0] = arg9[0] = 0;

/* parse user text */
sscanf(text, "%s %s %s %s %s %s %s %s %s",\
cmd, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);

/* default no command */
*buffer = 0;
if(strncasecmp(cmd, "highpower", 9) == 0) *buffer = HIGH_POWER; 
else if(strncasecmp(cmd, "lowpower", 8) == 0) *buffer = LOW_POWER; 
else if(strncasecmp(cmd, "reset", 5) == 0) *buffer = RESET; 
else if(strncasecmp(cmd, "compression", 11) == 0)
	{
	if(strncasecmp(arg1, "off", 3) == 0) *buffer = COMPRESSION_OFF;
	else if(strncasecmp(arg1, "on", 2) == 0) *buffer = COMPRESSION_ON;
	else return 0;
	}
else if(strncasecmp(cmd, "big", 3) == 0) *buffer = BIG_PICURE; 
else if(strncasecmp(cmd, "small", 5) == 0) *buffer = SMALL_PICURE; 
else if(strncasecmp(cmd, "auto", 4) == 0)
	{
	if(strncasecmp(arg1, "exposure", 8) == 0) *buffer = AUTO_EXPOSURE; 
	else if(strncasecmp(arg1, "white", 9) == 0) *buffer = AUTO_WHITE_BALANCE; 
	else
		{
		fprintf(stderr, "camera command syntax error\n");
		return 0;
		}
	}/* end if cmd is 'auto' */
else if(strncasecmp(cmd, "contrast", 8) == 0)
	{
	*buffer = CONTRAST;
	*(buffer + 1) = atoi(arg1) & 0xff;
	}
else if(strncasecmp(cmd, "brightness", 10) == 0)
	{
	*buffer = BRIGHTNESS;
	*(buffer + 1) = atoi(arg1) & 0xff;
	}
else if(strncasecmp(cmd, "saturation", 10) == 0)
	{
	*buffer = SATURATION;
	*(buffer + 1) = atoi(arg1) & 0xff;
	}
else if(strncasecmp(cmd, "man", 3) == 0)
	{
	if(strncasecmp(arg1, "white", 5) == 0)
		{
		*buffer = MAN_WHITE_BALANCE; 
		*(buffer + 1) = atoi(arg2) & 0xff;
		*(buffer + 2) = atoi(arg3) & 0xff;
		*(buffer + 3) = atoi(arg4) & 0xff;
		}/* end if man white */
	else if(strncasecmp(arg1, "exposure", 8) == 0)
		{
		*buffer = MAN_EXPOSURE;
		*(buffer + 1) = atoi(arg2) & 0xff;		/* gain */
		*(buffer + 2) = atoi(arg3) & 127;		/* fine 0 to 127 */
		*(buffer + 3) = atoi(arg4) / 256;		/* coarse high */
		*(buffer + 4) = atoi(arg4) % 256;		/* coarse low */
		}/* end if man exposure */
	else
		{
		fprintf(stderr, "camera command syntax error\n");
		return 0;
		}
	}/* end in cmd is 'man' */
else if(strncasecmp(cmd, "gain_ceiling", 11) == 0)
	{
	*buffer = GAIN_CEILING;
	*(buffer + 1) =  atoi(arg1) & 0x07; /* gain ceiling 0,1,2,3,4 */
	}
else if(strncasecmp(cmd, "compensation", 12) == 0)
	{
	*buffer = COMPENSATION_GAIN;
	*(buffer + 1) = atoi(arg1) & 0xff;		/* red compensation */
	*(buffer + 2) = atoi(arg2) & 0xff;		/* green 1 compensation */
	*(buffer + 3) = atoi(arg3) & 0xff;		/* green 2 compensation */
	*(buffer + 4) = atoi(arg4) & 0xff;		/* blue compensation */
	}
else if(strncasecmp(cmd, "restart", 7) == 0)
	{
	*buffer = CAMERA_RESTART;
	}
else if(strncasecmp(cmd, "coding_quality", 14) == 0)
	{
	a = atoi(arg1);
	if(a < 0)
		{
		fprintf(stderr,\
		"Using quality 0 as allowed minimum for client server operation.\n");
		a = 0;		
		}
	if(a > 100)
		{
		fprintf(stderr,\
		"Using quality 100 as allowed maximum for client server operation.\n");
		a = 100;		
		}

	*buffer = CODING_QUALITY;
	*(buffer + 1) = a;
	}
else if(strncasecmp(cmd, "coding", 6) == 0)
	{
	if(strncasecmp(arg1, "off", 3) == 0)
		{
		*buffer = NO_CODING;
		}
	else if(strncasecmp(arg1, "jpeg", 4) == 0)
		{
		*buffer = JPEG_CODING;
		}
	else if(strncasecmp(arg1, "mpeg", 4) == 0)
		{
		*buffer = MPEG2_CODING;
		}
	else if(strncasecmp(arg1, "wavelet", 4) == 0)
		{
		*buffer = WAVELET_CODING;
		}
	else
		{
		fprintf(stderr, "camera command syntax error\n");
		return 0;
		}
	}
else if(strcasecmp(cmd, "clock") == 0)
	{
	if(strcasecmp(arg1, "on") == 0) *buffer = CLOCK_ON;
	else if(strcasecmp(arg1, "off") == 0) *buffer = CLOCK_OFF;
	else fprintf(stderr, "command syntax error\n");
	}
else if(strcasecmp(cmd, "clock_position") == 0)
	{
	*buffer = CLOCK_POSITION;
	if(strcasecmp(arg1, "top_left") == 0)
		{
		*(buffer + 1) = TOP_LEFT;
		}
	else if(strcasecmp(arg1, "top_middle") == 0)
		{
		*(buffer + 1) = TOP_MIDDLE;
		}
	else if(strcasecmp(arg1, "top_right") == 0)
		{
		*(buffer + 1) = TOP_RIGHT;
		}
	else if(strcasecmp(arg1, "bottom_left") == 0)
		{
		*(buffer + 1) = BOTTOM_LEFT;
		}
	else if(strcasecmp(arg1, "bottom_middle") == 0)
		{
		*(buffer + 1) = BOTTOM_MIDDLE;
		}
	else if(strcasecmp(arg1, "bottom_right") == 0)
		{
		*(buffer + 1) = BOTTOM_RIGHT;
		}
	else
		{
		fprintf(stderr, "Command syntax error\n");
		}
	}/* end if clock position */
else if(strcasecmp(cmd, "time") == 0)
	{
	if(strcasecmp(arg1, "utc") == 0) *buffer = CLOCK_UTC;
	else if(strcasecmp(arg1, "local") == 0) *buffer = CLOCK_LOCAL;
	else fprintf(stderr, "command syntax error\n");
	}
else if(strcasecmp(cmd, "text") == 0)
	{
	if(strcasecmp(arg1, "screen") == 0) *buffer = TEXT_TO_SCREEN;
	else if(strcasecmp(arg1, "console") == 0) *buffer = TEXT_TO_CONSOLE;
	else fprintf(stderr, "command syntax error\n");
	}
else if(strcasecmp(cmd, "text_position") == 0)
	{
	*buffer = TEXT_POSITION;
	a = atoi(arg1);
	*(buffer + 1) = a / 256;
	*(buffer + 2) = a % 256;
	
	a = atoi(arg2);
	*(buffer + 3) = a / 256;
	*(buffer + 4) = a % 256; 
	}
else if(strcasecmp(cmd, "text_color") == 0)
	{
	*buffer = TEXT_COLOR;
	*(buffer + 1) = atoi(arg1);
	*(buffer + 2) = atoi(arg2);
	*(buffer + 3) = atoi(arg3);
	*(buffer + 4) = atoi(arg4);
	*(buffer + 5) = atoi(arg5);
	*(buffer + 6) = atoi(arg6);
	*(buffer + 7) = atoi(arg7);
	}
else if(strcasecmp(cmd, "characters") == 0)
	{
	if(strcasecmp(arg1, "normal") == 0) *buffer = CHAR_SET_S14;
	else if(strcasecmp(arg1, "umlaut") == 0) *buffer = CHAR_SET_TXT;
	}
else if(strcasecmp(cmd, "apcor") == 0)
	{
	*buffer = SET_APERTURE_CORRECTION;
	*(buffer + 1) = (16 * (atoi(arg1) & 0xf) ) + (atoi(arg2) & 0xf);
	*(buffer + 2) = (16 * (atoi(arg3) & 0xf) ) + (atoi(arg4) & 0xf);
	*(buffer + 3) = (16 * (atoi(arg5) & 0xf) ) + (atoi(arg6) & 0xf);
	*(buffer + 4) = (16 * (atoi(arg7) & 0xf) ) + (atoi(arg8) & 0xf);
	}
else if(strcasecmp(cmd, "apcor_default") == 0)
	{
	*buffer = SET_APCOR_DEFAULT;
	}
else if(strcasecmp(cmd, "sensor_fps") == 0)
	{
	*buffer = SET_SENSOR_FRAMES_PER_SECOND;
	*(buffer + 1) = atoi(arg1);
	}

/* CCCCCCCCC */
else /* not a valid command */
	{
	fprintf(stderr, "camera command syntax error\n");
	return 0;/* not a valid command */
	}

return 1;
}/* end function construct_camera_command */


int show_commands()
{
fprintf(stderr, "/camera highpower\n");
fprintf(stderr, "/camera lowpower\n");
fprintf(stderr, "/camera reset\n");
fprintf(stderr, "/camera compression off (or on)\n");
fprintf(stderr, "/camera big\n");
fprintf(stderr, "/camera small\n");
fprintf(stderr, "/camera auto exposure\n");
fprintf(stderr, "/camera auto white\n");
fprintf(stderr, "/camera contrast (0-100)\n");
fprintf(stderr, "/camera brightness (0-100)\n");
fprintf(stderr, "/camera saturation (0-100)\n");
fprintf(stderr, "/camera man exposure gain(0-8) fine_exp coarse_exp\n"); 
fprintf(stderr,\
"/camera compensation redcomp green1comp green2comp bluecomp (default 220 214 214 230\n");
fprintf(stderr, "/camera man white red(0-212)  green(0-212) blue(0-212)\n");
fprintf(stderr, "/camera gain_ceiling (0-4), (1x1) 2(x2) 3(x4) 4(x8)\n");
fprintf(stderr,\
"/camera restart (attempts reset, lowpower, highpower sequence)\n");
fprintf(stderr,\
"/camera coding (off, jpeg, mpeg, wavelet) selects none, jpeg, mpeg or wavelet\n");
fprintf(stderr,\
"/camera coding_quality (0 - 100) (usefull 25 - 80)\n");
fprintf(stderr,\
"/camera load name (sends commands to server from p.name in frame dir\n");
fprintf(stderr,\
"/camera save name (saves current settings in p.name in frame dir)\n");

fprintf(stderr,\
"/camera clock (on, off), displays a date and time on screen in the server\n");
fprintf(stderr,\
"/camera clock_position (top_left, top_middle, top_right,\n\
\tbottom_left, bottom_middle, bottom_right)\n");
fprintf(stderr,\
"/camera time (utc, local)\n"); 

fprintf(stderr,\
"/camera text (screen, console), text to screen or server console only\n");
fprintf(stderr,\
"/camera text_position x y  (in pixels), print position on screen text\n");
fprintf(stderr,\
"/camera text_color fg_red fg_green fg_blue bg_red bg_blue bg_green (0-255)\n");
fprintf(stderr,\
"/camera characters (normal, umlaut) (selects normal character set, or with umlauts)\n");
fprintf(stderr,\
"/camera apcor treshold1 intensity1 trehs2 intens2 thresh3 intens3 thres4 intens4\n");
fprintf(stderr,\
"/camera apcor_default\n\
(selects default aperture correction (1 12  1 10  2 13  2 10) (0-15 legal)\n");
fprintf(stderr,\
"/camera sensor_fps\n\
(sets server frames per second, slower more exposure time (3.125-30 legal)\n");
fprintf(stderr,\
"/local recording (on, off)  controls recording of picture (xbuffer or .ppm)\n");
fprintf(stderr,\
"/local frame_delay (0-...)  delay in MILLI seconds between frames requests\n");  
fprintf(stderr,\
"/local start_frame (0-...)  start frame numbering for recording\n");
fprintf(stderr,\
"/local frame_format (xbuffer, ppm, yuv), format in which frames are saved\n");
fprintf(stderr,\
"/local motion_detection (on, off), inhibits recording of frame if no motion\n");

fprintf(stderr,\
"/local clock (on, off), if on displays a date and time on screen\n");
fprintf(stderr,\
"/local clock_position (top_left, top_middle, top_right,\n\
\tbottom_left, bottom_middle, bottom_right)\n");
fprintf(stderr,\
"/local time (utc, local), clock uses Universal (GMT) or local time\n"); 

fprintf(stderr,\
"/local colordepth (auto, 16, 24, 32) for auto, 16, 24, or 32 bits per pixel\n");

fprintf(stderr, "/save filename, saves status in framedir/p.filename\n");
fprintf(stderr, "/load filename, load settings from framedir/p.filename\n");


fprintf(stderr, "/stat shows camera and server status\n");

/* AAAAAAAA */
}/* end function show_commands */


int save_status(BYTE __huge *buffer, FILE *fptr)
{
int a, b, b0, b1;

if(debug_flag)
	{
	fprintf(stderr, "save_status(): arg buffer=%lu, fptr=%lu\n",\
	buffer, fptr);
	}

if(! buffer) return 0;
if(! fptr) return 0;

if(*(buffer + FLAGS2) & 1) fprintf(fptr, "/camera big\n");
else fprintf(fptr, "/camera small\n");

/* this here, in case off command transfer is faster */
if(! *(buffer + FLAGS) & 2)
	{
	fprintf(fptr, "/camera coding off\n");
	}

b0 = *(buffer + FLAGS) & 2;
b1 = *(buffer + FLAGS) & 8;

if( !b1 && !b0 ) fprintf(fptr, "/camera coding off\n");
if( !b1 && b0 ) fprintf(fptr, "/camera coding jpeg\n");
if( b1 && !b0 ) fprintf(fptr, "/camera coding mpeg\n");
if( b1 && b0 ) fprintf(fptr, "/camera coding wavelet\n");

if(*(buffer + FLAGS2) & 2) fprintf(fptr, "/camera text screen\n");
else fprintf(fptr, "/camera text console\n");

if( *(buffer + HEADER_SIZE + 28) == 0)
	{
	fprintf(fptr, "/camera compression off\n");
	}
else
	{
	fprintf(fptr, "/camera compression on\n");
	}

if(*(buffer + HEADER_SIZE + 32) == 4)
	{
	fprintf(fptr, "/camera lowpower\n");
	}
else if(*(buffer + HEADER_SIZE + 32) == 3)
	{
	fprintf(fptr, "/camera highpower\n");
	}

fprintf(fptr, "/camera brightness %d\n", *(buffer + SERVER_STATUS + 11) );
fprintf(fptr, "/camera contrast %d\n", *(buffer + SERVER_STATUS + 12) );
fprintf(fptr, "/camera saturation %d\n", *(buffer + SERVER_STATUS + 13) );

/* if man_exposure_flag) */
if(*(buffer + FLAGS) & 16) /* man */
	{
	/* gain, fine, coarse */
	fprintf(fptr, "/camera man exposure %d %d %d\n",\
	*(buffer + SERVER_STATUS + 17),\
	*(buffer + SERVER_STATUS + 18),\
	*(buffer + SERVER_STATUS + 19) * 256 + *(buffer + SERVER_STATUS + 20) );
	}
else
	{
	fprintf(fptr, "/camera auto exposure\n");
	}

/* (1(x1) 2(x2) 3(x4) 4(x8) */
fprintf(fptr, "/camera gain_ceiling %d\n", *(buffer + SERVER_STATUS + 17) + 1);

/* red, green1, green2, blue */
fprintf(fptr, "/camera compensation %d %d %d %d\n",\
*(buffer + SERVER_STATUS + 21),\
*(buffer + SERVER_STATUS + 22), *(buffer + SERVER_STATUS + 23),\
*(buffer + SERVER_STATUS + 24) );

if(*(buffer +  FLAGS) & 32)
	{
	fprintf(fptr, "/camera auto white\n");
	}
else
	{
	/* red, green, blue */
	fprintf(fptr, "/camera man white %d %d %d\n",\
	*(buffer + SERVER_STATUS + 14),\
	*(buffer + SERVER_STATUS + 15),\
	*(buffer + SERVER_STATUS + 16) );
	}

fprintf(fptr, "/camera coding_quality %d\n",\
*(buffer + SERVER_STATUS + 33) );

/* server clock */
if(*(buffer + FLAGS) & 128) fprintf(fptr, "/camera time utc\n");
else fprintf(fptr, "/camera time local\n");

a = *(buffer + SERVER_STATUS + 34);
if(a == TOP_LEFT) fprintf(fptr, "/camera clock_position top_left\n");
if(a == TOP_MIDDLE) fprintf(fptr, "/camera clock_position top_middle\n");
if(a == TOP_RIGHT) fprintf(fptr, "/camera clock_position top_right\n");
if(a == BOTTOM_LEFT) fprintf(fptr, "/camera clock_position bottom_left\n");
if(a == BOTTOM_MIDDLE) fprintf(fptr, "/camera clock_position bottom_middle\n");
if(a == BOTTOM_RIGHT) fprintf(fptr, "/camera clock_position bottom_right\n");

if(*(buffer + FLAGS) & 64) fprintf(fptr, "/camera clock on\n");
else fprintf(fptr, "/camera clock off\n");

if(*(buffer + FLAGS2) & 2) fprintf(fptr, "/camera text screen\n");
else fprintf(fptr, "/camera text console\n");

if(*(buffer + FLAGS2) & 16) fprintf(fptr, "/camera characters umlaut\n");
else fprintf(fptr, "/camera characters normal\n");

a = *(buffer + SERVER_STATUS + 25);
if( *(buffer + FLAGS2) & 4) a += 256;

b = *(buffer + SERVER_STATUS + 26);
if( *(buffer + FLAGS2) & 8) b += 256;

fprintf(fptr, "/camera text_position %d %d\n", a, b);

fprintf(fptr, "/camera text_color %d %d %d  %d %d %d\n",\
*(buffer + SERVER_STATUS + 27),\
*(buffer + SERVER_STATUS + 28),\
*(buffer + SERVER_STATUS + 29),\
*(buffer + SERVER_STATUS + 30),\
*(buffer + SERVER_STATUS + 31),\
*(buffer + SERVER_STATUS + 32) );

fprintf(fptr, "/camera apcor %d %d  %d %d  %d %d  %d %d\n",\
(*(buffer + SERVER_STATUS + 0) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 0) & 0xf,
(*(buffer + SERVER_STATUS + 1) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 1) & 0xf,\
(*(buffer + SERVER_STATUS + 2) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 2) & 0xf,\
(*(buffer + SERVER_STATUS + 3) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 3) & 0xf);

fprintf(fptr, "/camera sensor_fps %d\n",\
*(buffer + SERVER_STATUS + 10) );

/* local clock */
if(use_utc_flag) fprintf(fptr, "/local time utc\n");
else fprintf(fptr, "/local time local\n");

a = clock_position;
if(a == TOP_LEFT) fprintf(fptr, "/local clock_position top_left\n");
if(a == TOP_MIDDLE) fprintf(fptr, "/local clock_position top_middle\n");
if(a == TOP_RIGHT) fprintf(fptr, "/local clock_position top_right\n");
if(a == BOTTOM_LEFT) fprintf(fptr, "/local clock_position bottom_left\n");
if(a == BOTTOM_MIDDLE) fprintf(fptr, "/local clock_position bottom_middle\n");
if(a == BOTTOM_RIGHT) fprintf(fptr, "/local clock_position bottom_right\n");

if(show_time_flag) fprintf(fptr, "/local clock on\n");
else fprintf(fptr, "/local clock off\n");

if(save_frames_format == 0)
	{
	fprintf(fptr, "/local frame_format xbuffer\n");
	}
else if(save_frames_format == 1)
	{
	fprintf(fptr, "/local frame_format ppm\n");
	}
else if(save_frames_format == 2)
	{
	fprintf(fptr, "/local frame_format yuv to stdout\n");
	}
else
	{
	fprintf(stderr,\
	"mcam: internet_client: save_status(): unknown save_frames_format=%d, aborting.\n", save_frames_format);

	exit(1);
	}

fprintf(fptr, "/local frame_delay %d\n", frame_delay);

fprintf(fptr, "/local start_frame %d\n", start_frame);

if(store_only_on_change_flag)
	{
	fprintf(fptr, "/local motion_detection on\n");
	}
else
	{
	fprintf(fptr, "/local motion_detection off\n");
	}

if(save_frames_flag)
	{
	fprintf(fptr, "/local recording on\n");
	}
else
	{
	fprintf(fptr, "/local recording off\n");
	}

if(color_depth_mode == 0)
	{
	fprintf(fptr, "/local colordepth auto\n");
	}
else
	{
	fprintf(fptr, "/local colordepth %d\n", color_depth);
	}

/* report to user what is happening */
fprintf(stderr, "Ready saving status\n");

fclose(fptr);

return 1;
}/* end function save_status */


int show_camera_status(BYTE __huge *buffer)
{
int a, b, b0, b1;
extern int color_depth;

/* 
Some things are taken from the transmit header, 
some from the camera frame header.
*/

/* server statistics */
if(*(buffer + FLAGS) & 4)
	{
	fprintf(stderr,\
	"server: f/s=%.2f  KB/s=%d.\n",\
	( (float)(*(buffer + SERVER_STATUS + 37) * 256) +\
	(float)*(buffer + SERVER_STATUS + 38) ) / 1000.0,\
	(*(buffer + SERVER_STATUS + 39) * 256) +\
	*(buffer + SERVER_STATUS + 40) );
	}
else
	{
	fprintf(stderr,\
	"server: f/s=%.2f  B/s=%d.\n",\
	( (float)(*(buffer + SERVER_STATUS + 37) * 256) +\
	(float)*(buffer + SERVER_STATUS + 38) ) / 1000.0,\
	(*(buffer + SERVER_STATUS + 39) * 256) +\
	*(buffer + SERVER_STATUS + 40) );
	}

fprintf(stderr, "clients connected to server=%d.\n",\
*(buffer + SERVER_STATUS + 35) );

/* server version */
a = *(buffer + SERVER_STATUS + 4);
fprintf(stderr, "server version=%d.%d\n",\
(a & 0xf0) >> 4, a & 0xf);

if(*(buffer + HEADER_SIZE + 32) == 4)
	{
	fprintf(stderr, "low power mode.");
	}
else if(*(buffer + HEADER_SIZE + 32) == 3)
	{
	fprintf(stderr, "high power mode.");
	}
fprintf(stderr, "\n");

/* video size */
if(*(buffer + FLAGS2) & 1) fprintf(stderr, "big picture 352 x 288.\n");
else fprintf(stderr, "small picture 176 x 144.\n");

if(*(buffer + FLAGS2) & 2) fprintf(stderr, "text to screen.\n");
else fprintf(stderr, "text to console.\n");

if(*(buffer + FLAGS2) & 16)
	{
	fprintf(stderr, "character set with umlauts selected.\n");
	}
else fprintf(stderr, "normal character set selected.\n");

a = *(buffer + SERVER_STATUS + 25);
if( *(buffer + FLAGS2) & 4) a += 256;

b = *(buffer + SERVER_STATUS + 26);
if( *(buffer + FLAGS2) & 8) b += 256;

fprintf(stderr, "camera text_position %d %d.\n", a, b);

fprintf(stderr, "camera text_color %d %d %d  %d %d %d.\n",\
*(buffer + SERVER_STATUS + 27),\
*(buffer + SERVER_STATUS + 28),\
*(buffer + SERVER_STATUS + 29),\
*(buffer + SERVER_STATUS + 30),\
*(buffer + SERVER_STATUS + 31),\
*(buffer + SERVER_STATUS + 32) );

/* compression */
if( *(buffer + HEADER_SIZE + 28) == 0)
	{
	fprintf(stderr, "compression is off.\n");
	}
if( *(buffer + HEADER_SIZE + 28) == 1)
	{
	fprintf(stderr, "compression is on.\n");
	fprintf(stderr, "compression ratio achieved for frame=%d\n.",\
	*(buffer + HEADER_SIZE + 40) );
	}
/*
fprintf(stderr, "frame rate achieved=%d.\n", *(buffer + HEADER_SIZE + 41) );
*/

fprintf(stderr, "brightness=%d  contrast=%d  saturation=%d.\n",\
*(buffer + SERVER_STATUS + 11), *(buffer + SERVER_STATUS + 12),\
*(buffer + SERVER_STATUS + 13) );

if(*(buffer +  FLAGS) & 32)
	{
	fprintf(stderr, "white balance is auto: ");
	}
else
	{
	fprintf(stderr, "white balance is manual: ");
	}

fprintf(stderr, "red gain=%d  green gain=%d  blue gain=%d.\n",
*(buffer + SERVER_STATUS + 14), *(buffer + SERVER_STATUS + 15),\
*(buffer + SERVER_STATUS + 16) );

fprintf(stderr, "gain_ceiling=%d.\n", *(buffer + SERVER_STATUS + 17) + 1);

/* exposure man/auto */
if(*(buffer + FLAGS) & 16) fprintf(stderr, "exposure is manual: ");
else fprintf(stderr, "exposure is auto: ");

fprintf(stderr, "exposure fine=%d  exposure coarse=%d.\n",
*(buffer + SERVER_STATUS + 18),\
*(buffer + SERVER_STATUS + 19) * 256 + *(buffer + SERVER_STATUS + 20) );

fprintf(stderr,\
"red comp=%d  green1 comp=%d  green2 comp=%d  blue comp=%d.\n",\
*(buffer + SERVER_STATUS + 21),\
*(buffer + SERVER_STATUS + 22), *(buffer + SERVER_STATUS + 23),\
*(buffer + SERVER_STATUS + 24) );

fprintf(stderr,\
"apertur treshhold1=%d level1=%d t2=%d l2=%d t3=%d l3=%d t4=%d l4=%d\n",\
(*(buffer + SERVER_STATUS + 0) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 0) & 0xf,\
(*(buffer + SERVER_STATUS + 1) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 1) & 0xf,\
(*(buffer + SERVER_STATUS + 2) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 2) & 0xf,
(*(buffer + SERVER_STATUS + 3) & 0xf0) >> 4,\
*(buffer + SERVER_STATUS + 3) & 0xf);

fprintf(stderr, "sensor_frames_per_second=%d\n",\
*(buffer + SERVER_STATUS + 10) );

if(save_frames_flag)
	{
	fprintf(stderr,\
	"local recording on  start_frame=%d  frame_delay=%d.\n",\
	start_frame, frame_delay);
	}
else
	{
	fprintf(stderr,\
	"local recording off  start_frame=%d  frame_delay=%d.\n",\
	start_frame, frame_delay);
	}

if(save_frames_format == 0)
	{
	fprintf(stderr, "save frames format is xbuffer.\n");
	}
else if(save_frames_format == 1)
	{
	fprintf(stderr, "save frames format is .ppm.\n");
	}
else if(save_frames_format == 2)
	{
	fprintf(stderr, "save frames format is yuv to stdout.\n");
	}
else
	{
	fprintf(stderr,\
	"mcam: internet_client: show_camera_status(): unknown save_frames_format=%d, aborting.\n",\
	save_frames_format);

	exit(1);
	}

if(store_only_on_change_flag)
	{
	fprintf(stderr, "motion detection is on.\n");
	}
else
	{
	fprintf(stderr, "motion detection is off.\n");
	}

/* coding */
b0 = *(buffer + FLAGS) & 2;
b1 = *(buffer + FLAGS) & 8;
if( !b1 && !b0) fprintf(stderr, "coding off.\n");
if( !b1 && b0) fprintf(stderr, "jpeg coding selected.\n");
if( b1 && !b0) fprintf(stderr, "mpeg2 coding selected.\n");
if( b1 && b0) fprintf(stderr, "wavelet coding selected.\n");

fprintf(stderr, "coding_quality=%d.\n", *(buffer + SERVER_STATUS + 33) );

/* server clock */
if(*(buffer + FLAGS) & 64)  fprintf(stderr, "camera clock display is on.\n");
else fprintf(stderr, "camera clock display is off.\n");

if(*(buffer + FLAGS) & 128) fprintf(stderr, "camera clock uses UTC (=GMT).\n");
else fprintf(stderr, "camera clock uses local time.\n");

a = *(buffer + SERVER_STATUS + 34);
if(a == TOP_LEFT) fprintf(stderr, "camera clock position top left.\n");
if(a == TOP_MIDDLE) fprintf(stderr, "camera clock position top middle.\n");
if(a == TOP_RIGHT) fprintf(stderr, "camera clock position top right.\n");
if(a == BOTTOM_LEFT) fprintf(stderr, "camera clock position bottom left.\n");
if(a == BOTTOM_MIDDLE) fprintf(stderr, "camera clock position bottom middle.\n");
if(a == BOTTOM_RIGHT) fprintf(stderr, "camera clock position bottom right.\n");

/* local clock */
if(show_time_flag) fprintf(stderr, "local clock display is on.\n");
else fprintf(stderr, "local clock display is off.\n");

if(use_utc_flag) fprintf(stderr, "local clock uses UTC (=GMT).\n");
else fprintf(stderr, "local clock uses local time.\n");

a = clock_position;
if(a == TOP_LEFT) fprintf(stderr, "local clock position top left.\n");
if(a == TOP_MIDDLE) fprintf(stderr, "local clock position top middle.\n");
if(a == TOP_RIGHT) fprintf(stderr, "local clock position top right.\n");
if(a == BOTTOM_LEFT) fprintf(stderr, "local clock position bottom left.\n");
if(a == BOTTOM_MIDDLE) fprintf(stderr, "local clock position bottom middle.\n");
if(a == BOTTOM_RIGHT) fprintf(stderr, "local clock position bottom right.\n");

if(color_depth_mode == 1)
	{
	fprintf(stderr, "colordepth=%d  X11 reported colordepth=%d\n",\
	color_depth, get_x11_color_depth() );
	}
else
	{
	fprintf(stderr, "colordepth=%d (auto)\n", color_depth);
	}

fprintf(stderr, "/help for list of commands.\n");

return 1;
}/* end function show_camera_status */


int execute_local_commands(char *text)
{
/*
the format is:
cmd, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8
*/
char cmd[80];
char arg1[80];
char arg2[80];
char arg3[80];
char arg4[80];
char arg5[80];
char arg6[80];
char arg7[80];
char arg8[80];
char arg9[80];

if(debug_flag)
	{
	fprintf(stderr, "execute_local_commands(): arg text=%s\n",\
	text);
	}

if(! text) return 0;

/* clear arguments */
cmd[0] = arg1[0] = arg2[0] = arg3[0] = arg4[0] = arg5[0] = arg6[0] =\
arg7[0] = arg8[0] = arg9[0] = 0;

/* parse user text */
sscanf(text, "%s %s %s %s %s %s %s %s %s",\
cmd, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);

if(strncasecmp(cmd, "recording", 9) == 0)
	{
	if(strncasecmp(arg1, "on", 2) == 0)
		{
		save_frames_flag = 1;
		fprintf(stderr, "recording on\n");
		}
	else
		{
		save_frames_flag = 0;
		fprintf(stderr, "recording off\n");
		}
	}
else if(strncasecmp(cmd, "frame_delay", 11) == 0)
	{
	frame_delay = atoi(arg1);
	fprintf(stderr, "frame_delay=%d milli seconds\n", frame_delay);
	}
else if(strncasecmp(cmd, "start_frame", 11) == 0)
	{
	start_frame = atoi(arg1);
	fprintf(stderr, "start_frame=%d\n", start_frame);
	frame_count = start_frame;
	}
else if(strncasecmp(cmd, "frame_format", 12) == 0)
	{
	if(strncasecmp(arg1, "ppm", 3) == 0)
		{
		save_frames_format = 1;
		fprintf(stderr, "frame_format is .ppm\n");
		}
	else if(strncasecmp(arg1, "yuv", 3) == 0)
		{
		save_frames_format = 2;
		fprintf(stderr, "frame_format is yuv to stdout\n");
		}
	else
		{
		save_frames_format = 0;
		fprintf(stderr, "frame_format is xbuffer\n");
		}
	}
else if(strncasecmp(cmd, "motion_detection", 16) == 0)
	{
	if(strncasecmp(arg1, "on", 2) == 0)
		{
		store_only_on_change_flag = 1;
		fprintf(stderr, "motion detection on\n");
		}
	else
		{
		store_only_on_change_flag = 0;
		fprintf(stderr, "motion detection off\n");
		}
	
	}
else if(strcasecmp(cmd, "clock") == 0)
	{
	if(strcasecmp(arg1, "on") == 0)
		{
		show_time_flag = 1;
		fprintf(stderr, "local clock on\n");
		}
	else
		{
		show_time_flag = 0;
		fprintf(stderr, "local clock off\n");
		}
	
	}
else if(strcasecmp(cmd, "clock_position") == 0)
	{
	if(strcasecmp(arg1, "top_left") == 0)
		{
		clock_position = TOP_LEFT;
		fprintf(stderr, "local clock position top left\n");
		}
	else if(strcasecmp(arg1, "top_middle") == 0)
		{
		clock_position = TOP_MIDDLE;
		fprintf(stderr, "local clock position top middle\n");
		}
	else if(strcasecmp(arg1, "top_right") == 0)
		{
		clock_position = TOP_RIGHT;
		fprintf(stderr, "local clock position top right\n");
		}
	else if(strcasecmp(arg1, "bottom_left") == 0)
		{
		clock_position = BOTTOM_LEFT;
		fprintf(stderr, "local clock position bottom left\n");
		}
	else if(strcasecmp(arg1, "bottom_middle") == 0)
		{
		clock_position = BOTTOM_MIDDLE;
		fprintf(stderr, "local clock position bottom middle\n");
		}
	else if(strcasecmp(arg1, "bottom_right") == 0)
		{
		clock_position = BOTTOM_RIGHT;
		fprintf(stderr, "local clock position bottom right\n");
		}
	else
		{
		fprintf(stderr, "Command syntax error\n");
		}
	}/* end if clock position */
else if(strcasecmp(cmd, "time") == 0)
	{
	if(strcasecmp(arg1, "utc") == 0)
		{
		use_utc_flag = 1;
		fprintf(stderr, "local clock uses UTC (=GMT)\n");
		}
	else /* local */
		{
		use_utc_flag = 0;
		fprintf(stderr, "local clock uses local time\n");
		}
	}
else if(strcasecmp(cmd, "colordepth") == 0)
	{
	if(strcasecmp(arg1, "auto") == 0)
		{
		color_depth_mode = 0;
		color_depth = get_color_depth();
		}
	else
		{
		color_depth = atoi(arg1);
		color_depth_mode = 1; /* manual */
		}
	}

/* BBBBBBBBB */
else /* invalid command */
	{
	fprintf(stderr, "local command syntax error\n");

	return 0;
	}

return 1;
}/* end function execute_local_commands */


